@@ -15,13 +15,7 @@ from ._field_data_comparison import (
     FieldComparison,
     FieldComparisonStatus,
     DefaultFieldComparisonCallback,
-    field_comparison_report
+    field_comparison_report,
 )
 
-__all__ = [
-    "FieldDataComparator",
-    "FieldComparisonSuite",
-    "FieldComparison",
-    "FieldComparisonStatus",
-    "__version__"
-]
+__all__ = ["FieldDataComparator", "FieldComparisonSuite", "FieldComparison", "FieldComparisonStatus", "__version__"]
