@@ -92,13 +92,9 @@ def accumulate(input_array: Array, axis: SupportsIndex = 0) -> Array:
     return np.sum(input_array, axis=axis)
 
 
-def adjacent_difference(input_array: Array, axis: int = -1) -> Array:
-    return np.diff(input_array, axis=axis)
-
-
-def all_true(input_array: Array, axis: Optional[SupportsIndex] = None):
-    """Check whether all entries of a boolean array are true along the given axis."""
-    return np.all(input_array, axis=axis)
+def get_adjacent_fuzzy_equal_indices(values: Array, abs_tol: float, rel_tol: float) -> Array:
+    result = np.isclose(values[:-1], values[1:], atol=abs_tol, rtol=rel_tol)
+    return append_to_array(result, False)
 
 
 def any_true(input_array: Array, axis: Optional[SupportsIndex] = None):
@@ -111,32 +107,43 @@ def append_to_array(input_array: Array, values) -> Array:
     return np.append(input_array, values)
 
 
-def elements_less(first: Array, second: Array) -> Array:
-    """Return a boolean array indicating entries of first that smaller than those of second."""
-    return np.less(first, second)
-
-
 def get_lex_sorting_index_map(input_array: Array) -> Array:
     """Get the list of indices for sorting the array lexicographically. Expects multi-dimensional arrays."""
     dimension = len(input_array[0])
     return np.lexsort(tuple(input_array[:, i] for i in reversed(range(dimension))))
 
 
+def get_fuzzy_lex_sorting_index_map(input_array: Array, abs_tol: float, rel_tol: float) -> Array:
+    """Get the list of indices for fuzzy-sorting the array lexicographically. Expects 2d arrays."""
+    if len(input_array.shape) != 2:
+        raise ValueError("Implementation only works for 2d arrays")
+    idx_map = np.argsort(input_array[:, 0])
+    sorted = input_array[idx_map]
+    for dim in range(1, input_array.shape[1]):
+        equals = get_adjacent_fuzzy_equal_indices(sorted[:, dim - 1], abs_tol=abs_tol, rel_tol=rel_tol)
+        for start, end in walk_adjacent_true_index_ranges(equals):
+            indices = np.argsort(sorted[start:end][:, dim])
+            idx_map[start:end] = idx_map[start:end][indices]
+            sorted[start:end] = input_array[idx_map[start:end]]
+    return idx_map
+
+
+def walk_adjacent_true_index_ranges(bool_array: Array, include_upper_edge: bool = True) -> Iterable[Tuple[int, int]]:
+    """Get an iterable over index chunks for which the given boolean array is true"""
+    begin, end, in_true_block = 0, 0, False
+    for i in range(len(bool_array)):
+        if bool_array[i] and not in_true_block:
+            begin, in_true_block = i, True
+        elif not bool_array[i] and in_true_block:
+            end, in_true_block = i, False
+            yield (begin, end + 1 if include_upper_edge else end)
+
+
 def get_sorting_index_map(input_array: Array) -> Array:
     """Get the list of indices that would sort the array."""
     return np.argsort(input_array)
 
 
-def abs_array(input_array: Array) -> Array:
-    """Return a copy of the array with the absolute values of the given array."""
-    return np.fabs(input_array)
-
-
-def min_element(input_array: Array):
-    """Return the minimum value within the array. Expects arrays of scalars."""
-    return input_array[np.argmin(input_array)]
-
-
 def max_element(input_array: Array):
     """Return the maximum value within the array. Expects arrays of scalars."""
     return input_array[np.argmax(input_array)]
@@ -188,6 +195,11 @@ def find_first_unequal(first: Array, second: Array) -> Optional[Tuple]:
     return None
 
 
+def fuzzy_equal(first: Array, second: Array, rel_tol: float, abs_tol: float) -> Array:
+    """Return the indices at which the two arrays are fuzzy_equal"""
+    return np.isclose(first, second, atol=abs_tol, rtol=rel_tol)
+
+
 def find_first_fuzzy_unequal(
     first: Array, second: Array, rel_tol: float = _default_base_tolerance(), abs_tol: float = _default_base_tolerance()
 ) -> Optional[Tuple]:
