@@ -1,6 +1,8 @@
 """Test field-IO from csv files"""
 
 from io import StringIO
+from csv import Error
+from pytest import raises
 
 from fieldcompare.predicates import ExactEquality
 from fieldcompare.tabular import Table, transform
@@ -40,6 +42,33 @@ def test_csv_field_extraction():
         )
 
 
+def test_csv_field_extraction_single_dtype():
+    reference_data = {
+        f"float_field_{i}": get_reference_data()["float_field"]
+        for i in range(3)
+    }
+    stream = _as_string_stream(reference_data, add_names=False)
+    fields = CSVFieldReader(delimiter=",", use_names=False, skip_rows=0).read(stream)
+
+    assert all(
+        ExactEquality()(
+            field.values,
+            get_reference_data()["float_field"]
+        ) for field in fields
+    )
+
+
+def test_csv_field_extraction_invalid_delimiter_raises_exception():
+    reference_data = {
+        f"float_field_{i}": get_reference_data()["float_field"]
+        for i in range(3)
+    }
+    stream = _as_string_stream(reference_data, add_names=True, delimiter="=")
+    with raises(Error) as e:
+        CSVFieldReader().read(stream)
+        assert "delimiter" in str(e)
+
+
 def test_csv_field_extraction_deduced_delimiters_and_headers():
     reference_data = get_reference_data()
     for delimiter in [",", ";", " "]:
